/*
 *  Copyright (c) 2010-2013, Rutgers, The State University of New Jersey
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *      * Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *      * Redistributions in binary form must reproduce the above copyright
 *        notice, this list of conditions and the following disclaimer in the
 *        documentation and/or other materials provided with the distribution.
 *      * Neither the name of the organization(s) stated above nor the
 *        names of its contributors may be used to endorse or promote products
 *        derived from this software without specific prior written permission.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 *  ARE DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 *  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file   mftimer.cpp
 * @author wontoniii (bronzino@winlab.rutgers.edu)
 * @date   October, 2013
 * @brief  Temp.
 *
 * Temp.
 */

#include <errno.h>
#include <time.h>

#include "mftimer.h"

#ifndef __ANDROID__
extern int errno;
#else
extern volatile int errno;
#endif




MF_Timer::MF_Timer() {
}

MF_Timer::MF_Timer(MF_EventQueue *e, int index, int t) {
	mTid = 0;
	sem_init(&access, 0, 1);
	trig = false;
	interface = index;
	running = false;
	eq = e;
	timeToSleepMicros = t;
	nextMicros = timeToSleepMicros;
}

MF_Timer::~MF_Timer() {
	
}

void MF_Timer::loop(){
	MF_Log::mf_log(MF_DEBUG, "MF_Timer:loop starting main loop");
	sem_wait(&access);
	trig = false;
	cont = true;
	running = true;
	nextMicros = timeToSleepMicros;
	while(cont){
		if(trig){
			MF_Log::mf_log(MF_DEBUG, "MF_Timer:loop event triggered sending csyn TO");
			MF_EvCysnTOut *to = new MF_EvCysnTOut(interface, chnkID);
			eq->add(to);
			running = false;
			cont = false;
		}
		else{
			MF_Log::mf_log(MF_DEBUG, "MF_Timer:loop no event triggered, I will sleep %d usecs", nextMicros);
			trig = true;
			gettimeofday(&started, NULL);
			sem_post(&access);
			// usleep(nextMicros);
			struct timespec req, rem;
			rem.tv_sec = nextMicros/1000000;
			rem.tv_nsec = (nextMicros%1000000)* 1000;
			while(1){
                            req = rem;
                            int ret = 0;
                            if(!(ret = nanosleep(&req, &rem)))break;
                            int errnum = errno;
                            if(errnum != EINTR){ 
			        MF_Log::mf_log(MF_ERROR, 
                                    "MF_Timer: nanosleep failed with errno %d",
                                    errnum);
                                break;
                            }
                            MF_Log::mf_log(MF_DEBUG, 
                                "MF_Timer: nanosleep interrupted, resuming");
                        }
			sem_wait(&access);
		}
	}
	running = false;
	sem_post(&access);
}

void MF_Timer::run() {
	int error = pthread_create(&mTid, NULL, &timerProc, this);
	if (error) MF_Log::mf_log(MF_ERROR, "Timer thread creation failed...\n");
}

void MF_Timer::reset(u_int chnkID){
	MF_Log::mf_log(MF_DEBUG, "MF_Timer:reset");
	sem_wait(&access);
	this->chnkID = chnkID;
	if(!running){
		MF_Log::mf_log(MF_DEBUG, "MF_Timer:reset - no timer running, started");
		run();
	}
	else{
		struct timeval now;
                //extend timer to duration passed uptill now
		gettimeofday(&now, NULL);
		nextMicros = 1000000*(int)(now.tv_sec - started.tv_sec);
                nextMicros += now.tv_usec - started.tv_usec;
		trig = false;
		cont = true;
		MF_Log::mf_log(MF_DEBUG, "MF_Timer:timer reinited");
	}
	sem_post(&access);
}

void MF_Timer::stop(){
	MF_Log::mf_log(MF_DEBUG, "MF_Timer:stop");
	sem_wait(&access);
	trig = false;
	cont = false;
	sem_post(&access);
}


u_int MF_Timer::getChunkID(){
	return chnkID;
}

void *timerProc(void *arg) {
	MF_Timer *t = (MF_Timer *)arg;
	t->loop();
	return NULL;
}
